﻿EnableExplicit

XIncludeFile "libcurl.pbi"

; PB already has such functions, just this one needs no release
; *str	:	a pointer to PB unicode string, or any memory containing unicode string
; RETURN:	PB unicode string with ascii bytes order inside, equal to array of 1-byte chars
Procedure$ ToAscii (*str)
	If *str 
		Protected str$ = PeekS(*str, #PB_Default, #PB_Unicode)
		Protected out$ = Space(1 + Len(str$) / SizeOf(Character))
		PokeS(@out$, str$, #PB_Default, #PB_Ascii)
		ProcedureReturn out$
	EndIf
EndProcedure

; libcurl sends responce data here
; RETURN:		size of processed data
ProcedureC CURLOPT_WRITEFUNCTION(*ptr, Size, NMemB, userdata)
	Protected *userdata.Integer = PeekI(userdata)
	Protected SizeProper = Size & 255
	Protected DataSize = SizeProper * NMemB
	Protected OldSize = *userdata\i
	Protected NewSize = OldSize + DataSize		; new buffer size needed to store data
	
	; enlarge buffer if needed
	If NewSize > MemorySize(*userdata)
		*userdata = ReAllocateMemory(*userdata, NewSize)
		PokeI(userdata, *userdata)
	EndIf
	
	; copy received data to a buffer
	CopyMemory(*ptr, *userdata + OldSize, DataSize)
	*userdata\i + DataSize
	
	ProcedureReturn DataSize
EndProcedure

; libcurl sends responce headers here
; RETURN:		size of processed data
ProcedureC CURLOPT_HEADERFUNCTION(*buffer, size, nitems, userdata)
	Protected *userdata.Integer = PeekI(userdata)
	Protected DataSize = nitems
	Protected OldSize = *userdata\i
	Protected NewSize = OldSize + DataSize		; new buffer size needed to store data
	
	; enlarge buffer if needed
	If NewSize > MemorySize(*userdata)
		*userdata = ReAllocateMemory(*userdata, NewSize)
		PokeI(userdata, *userdata)
	EndIf
	
	; copy received data to a buffer
	CopyMemory(*buffer, *userdata + OldSize, DataSize)
	*userdata\i + DataSize

	ProcedureReturn nitems
EndProcedure

; libcurl performs this callback when SSL/TLS connection initiates
; >The ssl_ctx parameter is actually a pointer to the SSL library's SSL_CTX for OpenSSL or wolfSSL
; RETURN:		none
ProcedureC CURLOPT_SSL_CTX_CALLBACK(*curl, *ssl_ctx, *clientp);
	; Here should be possible to load certs 
	; https://curl.se/libcurl/c/CURLOPT_SSL_CTX_FUNCTION.html
	Debug "CURLOPT_SSL_CTX_CALLBACK"
EndProcedure


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; some test
; using request to google DOH server
Global URL$ = ToAscii(@"https://dns.google.com/resolve?name=purebasic.com&type=A")
Global Proxy$ = ToAscii(@"socks4://127.0.0.1:8080")
Global curl = curl_easy_init()
Global curl_res

If curl
	; print curl version
	Debug PeekS(curl_version(), -1, #PB_UTF8)
	Debug ""

	; define request paramethers
	curl_easy_setopt(curl, #CURLOPT_URL, @URL$)				; target url
; 	curl_easy_setopt(curl, #CURLOPT_PROXY, @Proxy$)			; proxy
	curl_easy_setopt(curl, #CURLOPT_USERAGENT, @"TEST_UA")
	curl_easy_setopt(curl, #CURLOPT_FOLLOWLOCATION, 1)		; enable redirects
	curl_easy_setopt(curl, #CURLOPT_TIMEOUT, 30)			; set timeout
	
	
	; advanced
	curl_easy_setopt(curl, #CURLOPT_SSL_CTX_FUNCTION, @CURLOPT_SSL_CTX_CALLBACK()) ; SSL callback
	curl_easy_setopt(curl, #CURLOPT_SSL_VERIFYPEER, 0)		; 0 is bad here, but for now it doesn't work (another variant is using #CURLOPT_SSL_CTX_FUNCTION)
	curl_easy_setopt(curl, #CURLOPT_SSL_VERIFYHOST, 2)		;

	; callback to receive data
	Global *OutPointer.Integer = AllocateMemory(SizeOf(Integer))
	*OutPointer\i = MemorySize(*OutPointer)
	curl_easy_setopt(curl, #CURLOPT_WRITEFUNCTION, @CURLOPT_WRITEFUNCTION())
	curl_easy_setopt(curl, #CURLOPT_WRITEDATA, @*OutPointer)
	
	; callback to receive headers
	Global *OutPointerH.Integer = AllocateMemory(SizeOf(Integer))
	*OutPointerH\i = MemorySize(*OutPointerH)
	curl_easy_setopt(curl, #CURLOPT_HEADERFUNCTION, @CURLOPT_HEADERFUNCTION())
	curl_easy_setopt(curl, #CURLOPT_HEADERDATA, @*OutPointerH)

	; send request
	curl_res = curl_easy_perform(curl)

	; print received headers and data
	Debug PeekS(*OutPointerH+SizeOf(Integer), *OutPointerH\i-SizeOf(Integer), #PB_UTF8)
	Debug PeekS(*OutPointer+SizeOf(Integer), *OutPointer\i-SizeOf(Integer), #PB_UTF8)
	
	; cls
	curl_easy_cleanup(curl)
EndIf
